#ifndef __LIST_H__
#define __LIST_H__

#include <stdio.h>
#include "defns.h"

/**
 * A list node.
 * Contains the data and a pointer to the next node.
 */
typedef struct _ListNode {
	Type data;					// Our data.
	struct _ListNode *next;		// A pointer to the next element in the list. NULL if this is the last node.
} ListNode;

/**
 * Typedef a new name for a pointer to a ListNode.
 */
typedef ListNode * ListNodePtr;


/**
 * A List.
 * Contains the size of the list and a pointer to the first node. If this is NULL, then the list is empty.
 */
typedef struct _List {
	int size;				// The size of the list.
	ListNode *firstNode;	// A pointer to the first node in the list.
} List;

/**
 * Typedef a new name for a pointer to a List.
 */
typedef List * ListPtr;

/**
 * Forward Function Declarations.
 */

/// Returns TRUE if the list is empty.
BOOL listIsEmpty(List *list);

/// Initialize the list.
void initList(List *list);

/// Print the contents of the list.
void printList(List *list);

/// Insert data into the list.
void insertData(List *list, Type data);

/// Delete the first occuerence of data from the list. If deletion was succesfull, return TRUE, else FALSE.
BOOL deleteData(List *list, Type data);

/// Access the data at position index. Return NULL if unsuccesfull.
ListNode *accessNode(List *list, int index);

/// Clear the contents of the list.
void clearList(List *list);

/// Find the first occurence of the list node with value key, NULL if non existant.
ListNode *findNode(List *list, Type key);

/// Check if a value is in the list, NULL if non existant.
BOOL exists(List *list, Type key);

#endif // __LIST_H__