#include <stdio.h>

#include "MatrixGraph.h"
#include "ListGraph.h"

void runListGraphTest(void);
void runMatrixGraphTest(void);

int main(void)
{
	fprintf(stdout, "ADJACENCY MATRIX\n");
	runMatrixGraphTest();

	fprintf(stdout, "\n\nADJACENCY LIST\n");
	runListGraphTest();

	return 0;
}

// Run the test for the adjacency list graph
void runListGraphTest(void)
{	
	ListGraph *graph = lgCreate(UNDIRECTED, 6);
	
	// Create a graph
	lgAddEdge(graph, 0, 4, 1);
	lgAddEdge(graph, 0, 3, 1);
	lgAddEdge(graph, 0, 1, 1);
	lgAddEdge(graph, 1, 5, 1);
	lgAddEdge(graph, 1, 4, 1);
	lgAddEdge(graph, 1, 2, 1);
	lgAddEdge(graph, 1, 0, 1);
	lgAddEdge(graph, 2, 5, 1);
	lgAddEdge(graph, 2, 1, 1);
	lgAddEdge(graph, 3, 4, 1);
	lgAddEdge(graph, 3, 0, 1);
	lgAddEdge(graph, 4, 5, 1);
	lgAddEdge(graph, 4, 3, 1);
	lgAddEdge(graph, 4, 1, 1);
	lgAddEdge(graph, 4, 0, 1);
	lgAddEdge(graph, 5, 4, 1);
	lgAddEdge(graph, 5, 2, 1);
	lgAddEdge(graph, 5, 1, 1);
	
	// Print the graph
	fprintf(stdout, "Graph Structure:\n");
	fprintf(stdout, "----------------\n");
	lgPrint(graph);

	// Visit the nodes of the graph in Depth First Order
	fprintf(stdout, "DFS Order: ");
	lgDepthFirstSearch(graph, 0);
	fprintf(stdout, "BFS Order: ");
	lgBreadthFirstSearch(graph, 0);

	lgDelete(graph);
}

// Run the test for the adjacency matrix graph
void runMatrixGraphTest(void)
{
	MatrixGraph *graph = mgCreate(UNDIRECTED, 6);

	// Create a graph
	mgAddEdge(graph, 0, 1, 1);
	mgAddEdge(graph, 0, 3, 1);
	mgAddEdge(graph, 0, 4, 1);
	mgAddEdge(graph, 1, 0, 1);
	mgAddEdge(graph, 1, 2, 1);
	mgAddEdge(graph, 1, 4, 1);
	mgAddEdge(graph, 1, 5, 1);
	mgAddEdge(graph, 2, 1, 1);
	mgAddEdge(graph, 2, 5, 1);
	mgAddEdge(graph, 3, 0, 1);
	mgAddEdge(graph, 3, 4, 1);
	mgAddEdge(graph, 4, 0, 1);
	mgAddEdge(graph, 4, 1, 1);
	mgAddEdge(graph, 4, 3, 1);
	mgAddEdge(graph, 4, 5, 1);
	mgAddEdge(graph, 5, 1, 1);
	mgAddEdge(graph, 5, 2, 1);
	mgAddEdge(graph, 5, 4, 1);

	// Print the graph
	fprintf(stdout, "Graph Structure:\n");
	fprintf(stdout, "----------------\n");
	mgPrint(graph);

	// Visit the nodes of the graph in Depth First Order
	fprintf(stdout, "DFS Order: ");
	mgDepthFirstSearch(graph, 0);
	fprintf(stdout, "BFS Order: ");
	mgBreadthFirstSearch(graph, 0);

	mgDelete(graph);
}